package imagebuilder

const (
	// in docker/system
	NoBaseImageSpecifier = "scratch"

	// not yet part of our import
	commandArg        = "arg"
	commandStopSignal = "stopsignal"

	// in docker/system
	defaultPathEnv = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
)
